# -*- coding: utf-8 -*-
 
# Copyright (c) 2010, Dan Eicher.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#   * Redistributions of source code must retain the above copyright notice,
#     this list of conditions and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright notice,
#     this list of conditions and the following disclaimer in the documentation
#     and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
# <pep8 compliant>

bl_info = {
    "name": "Pie Library",
    "author": "Dan Eicher, Sean Olson",
    "version": (0, 1, 0),
    "blender": (2, 6, 4),
    "location": "",
    "description": "Library Used by Pie Menus",
    "warning": "",
    "wiki_url": "",
    "tracker_url": "",
    "category": "3D View"
}
 
import bpy
import math
import blf
import bgl

class MenuItem(object):
    def __init__(self, id, x=0, y=0, angle=0):
        self.id = id
        self.x = x
        self.y = y
        self.angle = angle
 
    def op(self, parent, context):
        raise NotImplementedError
 
    def poll(self, context):
        return True
 
class PieMenu(object):
    
    def __init__(self, x, y, layout_radius, text_size,
                 text_dpi, center_radius_squared,
                 max_radius_squared, corner_radius=5):
        self.menu_x = x
        self.menu_y = y
        self.radius = layout_radius
        self.text_size = text_size
        self.text_dpi = text_dpi
        self.center_radius = center_radius_squared
        self.max_radius = max_radius_squared
        self.corner_radius = corner_radius
        self.angle = 0
        self.item_offset = 0
        self.menu_items = []
        self.vec_cache = None

        ui=bpy.context.user_preferences.themes['Default'].user_interface #alias
        self.ris = ui.wcol_pulldown.inner_sel[0]
        self.gis = ui.wcol_pulldown.inner_sel[1]
        self.bis = ui.wcol_pulldown.inner_sel[2]
        self.ais = ui.wcol_pulldown.inner_sel[3]
    
        self.ri = ui.wcol_menu.inner[0]
        self.gi = ui.wcol_menu.inner[1]
        self.bi = ui.wcol_menu.inner[2]
        self.ai = ui.wcol_menu.inner[3]

        self.ro = ui.wcol_menu.outline[0]
        self.go = ui.wcol_menu.outline[1]
        self.bo = ui.wcol_menu.outline[2]

        self.rt = ui.wcol_menu.text[0]
        self.gt = ui.wcol_menu.text[1]
        self.bt = ui.wcol_menu.text[2] 
    
        self.pointerangle=0
        

    ## auto-layout menu items around the base circle
    def layout(self, degrees):
        angle = self.angle
        items = len(self.menu_items)
        if angle:  # keep sub-menu items inside main slice
            step = degrees / (items - 1)
            angle -= (degrees * 0.5)
        else:
            step = degrees / items
        self.item_offset = step * 0.5
 
        # Calculate pie slice
        for it in self.menu_items:
            it.angle = angle
 
            # Rotate point around center with Fancy Math
            it.x = self.radius * math.sin(math.radians(angle + it.bias))
            it.y = self.radius * math.cos(math.radians(angle + it.bias))
 
            angle += step
 
    ## fetch the item under the pointer
    def calc(self, x, y):
        # mouse position converted from screen space to menu space
        x -= self.menu_x
        y -= self.menu_y
 
        # Cursor in inactive regions
        rad = pow(x, 2) + pow(y, 2)
        if rad < self.center_radius:
            return None
        if rad > self.max_radius:
            return -1
 
        angle = (math.degrees(math.atan2(x, y)) - 360) % 360
        
        for it in self.menu_items:
            if (it.angle - self.item_offset) < angle <= (it.angle + self.item_offset):
                return it
 
        # special case for 12 o'clock position
        if angle <= self.item_offset or angle > (360 - self.item_offset):
            return self.menu_items[0]
        else:  # should never make it this far but you never know
            return None
 
    # draw the menu
    def draw(self, parent, context):

        biggestdimensionX=biggestdimensionY=0
        for it in self.menu_items:
            #find the biggest word in the menu and base the size of all buttons on that word
            blf.size(0, self.text_size, self.text_dpi)
            dimension = blf.dimensions(0, it.id)
    
            if dimension[0]>biggestdimensionX:
                biggestdimensionX = dimension[0]
            if dimension[1]>biggestdimensionY:
                biggestdimensionY = dimension[1]

        for it in self.menu_items:
            sel = (it == parent.current)
            it_poll = it.poll(context)
 
            # center item on the circle
            #x = (self.menu_x + it.x) - (dimension[0] * 0.5)
            #y = (self.menu_y + it.y) - (dimension[1] * 0.5)

            blf.size(0, self.text_size, self.text_dpi)
            dimension = blf.dimensions(0, it.id)

            #needed for box centering
            x = (self.menu_x + it.x) - (biggestdimensionX * 0.5)
            y = (self.menu_y + it.y) - (biggestdimensionY * 0.5)

            #needed offset for text centering
            blf.size(0, self.text_size, self.text_dpi)
            dimension = blf.dimensions(0, it.id)
            xt = ((biggestdimensionX-dimension[0]) * 0.5)
            yt = ((biggestdimensionY-dimension[1]) * 0.5)

            # Draw background buttons
            if sel and it_poll:
                bgl.glColor4f(self.ris, self.gis, self.bis, self.ais)
            else:
                bgl.glColor4f(self.ri, self.gi, self.bi, self.ai)
 
            #box background
            self._round_box(bgl.GL_POLYGON, x - 20, y - 5, x + biggestdimensionX + 20, y + biggestdimensionY + 5)
                
            #box outline 
            bgl.glColor4f(self.ro, self.go, self.bo, 1.0)
            
            self._round_box(bgl.GL_LINE_LOOP, x - 20, y - 5, x + biggestdimensionX + 20, y + biggestdimensionY + 5)

            bgl.glColor4f(self.ri, self.gi, self.bi, self.ai)

            #draw the circle
            self._circle_(bgl.GL_TRIANGLE_FAN, (self.menu_x), (self.menu_y), 20)
            
            #draw the circle outline
            bgl.glColor4f(self.ro, self.go, self.bo, 1.0)
            self._circle_(bgl.GL_LINE_LOOP, (self.menu_x), (self.menu_y), 20)

            self._pointer_(bgl.GL_TRIANGLE_STRIP, (self.menu_x), (self.menu_y), self.pointerangle )
 
            # Draw text
            blf.enable(0, blf.SHADOW)
            blf.shadow_offset(0, 0, 0)
            blf.shadow(0, 5, 0.0, 0.0, 0.0, 1.0)
            if it_poll:
                bgl.glColor3f(self.rt, self.gt, self.bt)
            else:  # grayed out
                bgl.glColor3f(0.5, 0.5, 0.5)
            blf.position(0, x+xt, y+yt, 0)
            blf.draw(0, it.id)
            blf.disable(0, blf.SHADOW)
 
    ## nabbed from interface_draw.c

    def _circle_(self, mode, centerX, centerY, radius):
        bgl.glBegin(mode)
        for angle in range(0,20):
            angle = angle*2*math.pi/20 
            bgl.glVertex2f(centerX + (math.cos(angle) * radius), centerY + (math.sin(angle) * radius))
        bgl.glEnd()

    def _line_(self, mode, startX, startY, endX, endY):
        bgl.glBegin(bgl.GL_LINES)
        bgl.glVertex2f(startX, startY)
        bgl.glVertex2f(endX, endY)
        bgl.glEnd()

    def _pointer_(self, mode, centerX, centerY, rotation):
        bgl.glBegin(mode);
        bgl.glColor4f(self.ris, self.gis, self.bis, self.ais)

        def draw_rotated(cos,sin, centerx, centery, x, y):
            nx = centerx + ((x * cos) - (y * sin))
            ny = centery + ((x * sin) + (y * cos))
            bgl.glVertex2f(nx,ny)

        sin = math.sin(math.radians(rotation))
        cos = math.cos(math.radians(rotation))

        draw_rotated(cos, sin, centerX, centerY, 0, 24)
        draw_rotated(cos, sin, centerX, centerY, -5, 14)
        draw_rotated(cos, sin, centerX, centerY, 5, 14)
        draw_rotated(cos, sin, centerX, centerY, -3, 10)
        draw_rotated(cos, sin, centerX, centerY, 3, 10)


        bgl.glPopMatrix()
        bgl.glEnd()


    def _round_box(self, mode, minx, miny, maxx, maxy):
       
        rad = self.corner_radius
        vec = [[0.195, 0.02],
               [0.383, 0.067],
               [0.55, 0.169],
               [0.707, 0.293],
               [0.831, 0.45],
               [0.924, 0.617],
               [0.98, 0.805]]
 
        # cache vec list to save 224 multiplications per 8 item menu redraw
        if self.vec_cache == None:
            self.vec_cache = [(point[0] * rad, point[1] * rad) for point in vec]
 
        bgl.glBegin(mode)
 
        # start with corner right-bottom
        bgl.glVertex2f(maxx - rad, miny)
        bgl.glVertex2fv(bgl.Buffer(bgl.GL_FLOAT, [7,2],
                                   [(maxx - rad + point[0], miny + point[1]) for point in self.vec_cache]))
        bgl.glVertex2f(maxx, miny + rad)
 
        # corner right-top
        bgl.glVertex2f(maxx, maxy - rad)
        bgl.glVertex2fv(bgl.Buffer(bgl.GL_FLOAT, [7,2],
                                   [(maxx - point[1], maxy - rad + point[0]) for point in self.vec_cache]))
        bgl.glVertex2f(maxx - rad, maxy)
 
        # corner left-top
        bgl.glVertex2f(minx + rad, maxy)
        bgl.glVertex2fv(bgl.Buffer(bgl.GL_FLOAT, [7,2],
                                   [(minx + rad - point[0], maxy - point[1]) for point in self.vec_cache]))
        bgl.glVertex2f(minx, maxy - rad)
 
        # corner left-bottom
        bgl.glVertex2f(minx, miny + rad)
        bgl.glVertex2fv(bgl.Buffer(bgl.GL_FLOAT, [7,2],
                                   [(minx + point[1], miny + rad - point[0]) for point in self.vec_cache]))
        bgl.glVertex2f(minx + rad, miny)
 
        bgl.glEnd()



